@@ -12,11 +12,15 @@
 val rset : string -> string -> unit
 val rget : string -> string
 val bset : string -> string -> unit
-val bget : bool -> string -> string
+val bget : bool -> string -> string option
 val init : string -> unit
+val final : string -> unit
 val finalize : bool -> bool
 val bwrite : string -> string -> unit
 val rwrite : string -> string -> unit
+val swrite : string -> unit
+
+val addtoc : string -> int -> string -> unit
 
 val hot_start : unit -> unit
 
