@@ -9,7 +9,7 @@
 (*                                                                     *)
 (***********************************************************************)
 
-let header = "$Id: html.ml,v 1.109 2006-10-11 16:43:20 maranget Exp $" 
+let header = "$Id: html.ml,v 1.114 2007/03/09 13:23:52 maranget Exp $" 
 
 (* Output function for a strange html model :
      - Text elements can occur anywhere and are given as in latex
@@ -104,11 +104,11 @@ let
 let set_out out =  !cur_out.out <- out
 
 and stop () =
-  Stack.push stacks.s_active !cur_out.out ;
+  MyStack.push stacks.s_active !cur_out.out ;
   !cur_out.out <- Out.create_null ()
 
 and restart () =
-  !cur_out.out <- Stack.pop stacks.s_active
+  !cur_out.out <- MyStack.pop stacks.s_active
 ;;
 
 
@@ -138,15 +138,7 @@ let put_char c =
   else HtmlCommon.put_char c
 ;;
 
-let put_unicode i = match i with
-| 0x3C -> put "&lt;"
-| 0x3E -> put "&gt;"
-| 0x26 -> put "&amp;"
-| _ ->
-  try put_char (OutUnicode.translate_out i)
-  with OutUnicode.CannotTranslate ->
-    put (Printf.sprintf "&#X%X;" i)
-;;
+let put_unicode i = OutUnicode.html_put put put_char i
 
 let loc_name _ = ()
 
@@ -186,8 +178,8 @@ let finalize check =
   end else begin
     (* Flush output in case of fatal error *)
     let rec close_rec () =
-      if not (Stack.empty out_stack) then begin
-        match Stack.pop out_stack with
+      if not (MyStack.empty out_stack) then begin
+        match MyStack.pop out_stack with
         | Freeze _ -> close_rec ()
         | Normal (_,_,pout) ->
             Out.copy !cur_out.out pout.out ;
@@ -214,10 +206,10 @@ let put_tag tag = put tag
 ;;
 
 let put_nbsp () =
-  if !Lexstate.whitepre || (flags.in_math && !Parse_opts.mathml) then
+  if !Lexstate.whitepre || (flags.in_math && !Parse_opts.mathml) then begin
     put_char ' '
-  else
-    put_unicode 0xA0
+  end else
+    put_unicode OutUnicode.nbsp
 ;;
 
 let put_open_group () =
@@ -290,7 +282,7 @@ let rec do_close_par () = match pblock () with
 | P ->
     ignore (close_flow_loc check_blank P) ;
     true
-| s ->
+| _ ->
     false
 
 
@@ -455,6 +447,7 @@ and insert_block s attr =
 and insert_attr s = insert_attr (find_block s)
 and erase_mods = erase_mods
 and open_mod = open_mod
+and has_mod = has_mod
 and clearstyle = clearstyle
 and nostyle = nostyle
 and get_fontsize = get_fontsize
