@@ -8,31 +8,29 @@
 (*  Automatique.  Distributed only by permission.                      *)
 (*                                                                     *)
 (***********************************************************************)
+
 open Lexstate
-
-    exception Error of string
-    type block
-
-    val iso : char -> string
-    val iso_string : string -> string
-
-    val set_out : Out.t -> unit
-    val stop : unit -> unit
-    val restart : unit -> unit
-    val get_last_closed : unit -> block
-    val set_last_closed : block -> unit
-    val is_empty : unit -> bool
-  
-    val get_fontsize : unit -> int
-    val nostyle : unit -> unit
-    val clearstyle : unit -> unit
-    val open_mod : Element.text -> unit
-    val erase_mods : Element.text list -> unit
-    val par : int option -> unit
+exception Error of string
+type block
+val set_out : Out.t -> unit
+val stop : unit -> unit
+val restart : unit -> unit
+val is_empty : unit -> bool
+    
+val get_fontsize : unit -> int
+val nostyle : unit -> unit
+val clearstyle : unit -> unit
+val open_mod : Element.text -> unit
+val erase_mods : Element.text list -> unit
+val has_mod : Element.text -> bool
     val forget_par : unit -> int option
+    val close_par : unit -> bool
+    val open_par : unit -> unit
+    val par : int option -> unit
     val open_block : string -> string -> unit
     val close_block : string -> unit
     val force_block : string -> string -> unit
+    val close_flow : string -> unit
     val insert_block : string -> string -> unit
     val insert_attr : string -> string -> unit
 
@@ -44,8 +42,6 @@ open Lexstate
     val item_display : unit -> unit
     val force_item_display : unit -> unit
     val erase_display : unit -> unit
-    val box_around_display : (string arg -> unit) -> string arg -> unit
-
     val standard_sup_sub :
         (string arg -> unit) -> (unit -> unit) -> string arg -> string arg -> bool -> unit
     val limit_sup_sub :
@@ -53,16 +49,14 @@ open Lexstate
     val int_sup_sub :
         bool -> int ->
           (string arg -> unit) -> (unit -> unit) -> string arg -> string arg -> bool -> unit
-
-    val over : bool -> Lexing.lexbuf -> unit
-    (* For use by \cfrac, \xleftarrow and \xrightarrow *)
-    val over_align : bool -> bool -> bool -> Lexing.lexbuf -> unit
-    val left : string -> (int -> unit) -> unit
-    val right : string -> int
+    val addvsize : int -> unit
+    val over : Lexing.lexbuf -> unit
+    val left : string -> (int -> unit) -> (int -> unit) -> unit
+    val right : string -> (int -> unit) -> int
 
     val set_dcount : string -> unit
-    val item : string -> unit
-    val nitem : string -> unit
+    val item : string-> unit
+    val nitem : string-> unit
     val ditem : (string -> unit) -> string -> string -> string -> unit
     val erase_block : string -> unit
     val open_group : string -> unit
@@ -70,6 +64,7 @@ open Lexstate
     val close_group : unit -> unit
     val put : string -> unit
     val put_char : char -> unit
+    val put_unicode : OutUnicode.unichar -> unit
     val flush_out : unit -> unit
     val skip_line : unit -> unit
 
@@ -92,7 +87,6 @@ open Lexstate
     val put_close_group : unit -> unit
     val put_in_math : string -> unit
 
-
     val open_table : bool -> string -> unit
     val new_row : unit -> unit
     val open_cell : Tabular.format -> int -> int -> unit
@@ -118,4 +112,3 @@ open Lexstate
     type saved 
     val check : unit -> saved
     val hot : saved -> unit
-
