@@ -11,6 +11,7 @@
 
 type t
 
+val free : t -> unit
 val create_buff : unit -> t
 val create_chan : out_channel -> t
 val create_null : unit -> t
@@ -21,10 +22,14 @@ val reset : t -> unit
 val is_empty: t -> bool
 
 val put : t -> string -> unit
+val blit : t -> Lexing.lexbuf -> unit
 val put_char : t -> char -> unit
 val flush: t -> unit
+val get_pos : t -> int
+val erase_start : int -> t -> unit
 
 val iter : (char -> unit) -> t -> unit
+val iter_next : (char -> (unit -> int) -> unit) -> t -> unit
 val to_string : t -> string
 val to_chan : out_channel -> t -> unit
 val copy : t -> t -> unit
