@@ -9,7 +9,7 @@
 (*                                                                     *)
 (***********************************************************************)
 
-(*  $Id: package.ml,v 1.96 2006-10-16 08:57:17 maranget Exp $    *)
+(*  $Id: package.ml,v 1.107 2007/06/14 14:34:22 maranget Exp $    *)
 
 module type S = sig  end
 
@@ -22,7 +22,7 @@ open Lexing
 open Lexstate
 open Latexmacros
 open Subst
-open Stack
+open MyStack
 open Scan
 ;;
 
@@ -40,14 +40,13 @@ open Scan
 (*********************************************************)
 
 let put_empty empty =
-  if empty <> 0 then Dest.put_unicode empty
+  if empty <> OutUnicode.null then Dest.put_unicode empty
   else raise OutUnicode.CannotTranslate 
 
-exception DiacriticFailed of string
+exception DiacriticFailed of string * string
 
-let do_def_diacritic verb name f empty = 
-  (fun lexbuf ->
-    Save.start_echo () ;
+let do_def_diacritic _verb _name f empty = 
+  (fun lexbuf ->    
     let arg0 = save_arg lexbuf in
     let arg = get_prim_onarg arg0 in
     try match String.length arg with
@@ -59,38 +58,41 @@ let do_def_diacritic verb name f empty =
     | _ -> raise OutUnicode.CannotTranslate
     with
     | OutUnicode.CannotTranslate
-    | Misc.CannotPut ->	raise (DiacriticFailed arg))
+    | Misc.CannotPut ->	raise (DiacriticFailed (Subst.do_subst_this arg0,arg)))
 
 let def_diacritic name internal f empty =
   def_code name
     (fun lexbuf ->
       try do_def_diacritic true name f empty lexbuf
-      with DiacriticFailed input ->
+      with DiacriticFailed (p,input) ->
 	scan_this main
-	  ("\\text@accent{"^internal^"}{"^name^"}{"^input^"}"))
+	  ("\\text@accent{"^internal^"}{"^name^"}{\\@print{"^input^"}}{"^p^"}"))
 ;;
 
-def_diacritic "\\'"  "acute" OutUnicode.acute 0xB4 ;
-def_diacritic "\\`"  "grave" OutUnicode.grave 0x60 ;
-def_diacritic "\\^"  "circumflex" OutUnicode.circumflex 0x5E ;
-def_diacritic "\\\"" "diaeresis" OutUnicode.diaeresis 0xA8 ;
-def_diacritic "\\c"  "cedilla" OutUnicode.cedilla 0xB8 ;
-def_diacritic "\\~"  "tilde" OutUnicode.tilde 0x7E ;
-def_diacritic "\\="  "macron" OutUnicode.macron 0xAF ;
-def_diacritic "\\H"  "doubleacute" OutUnicode.doubleacute 0x2DD ;
+open OutUnicode
+
+let () =
+  def_diacritic "\\'"  "acute" OutUnicode.acute acute_alone ;
+  def_diacritic "\\`"  "grave" OutUnicode.grave grave_alone ;
+  def_diacritic "\\^"  "circumflex" OutUnicode.circumflex circum_alone ;
+  def_diacritic "\\\"" "diaeresis" OutUnicode.diaeresis diaeresis_alone ;
+  def_diacritic "\\c"  "cedilla" OutUnicode.cedilla cedilla_alone ;
+  def_diacritic "\\~"  "tilde" OutUnicode.tilde tilde_alone ;
+  def_diacritic "\\="  "macron" OutUnicode.macron macron_alone ;
+  def_diacritic "\\H"  "doubleacute" OutUnicode.doubleacute doubleacute_alone ;
 (* package fc, see later ? *)
 (* def_diacritic "\\G"  "doublegrave" OutUnicode.doublegrave 0x2F5 ; *)
-def_diacritic "\\u"  "breve" OutUnicode.breve 0x2D8 ;
-def_diacritic "\\."  "dotabove" OutUnicode.dotabove 0x2D9 ;
-def_diacritic "\\d"  "dotbelow" OutUnicode.dotbelow 0x2D4 ;
-def_diacritic "\\b"  "linebelow" OutUnicode.linebelow 0x5F ;
-def_diacritic "\\k"  "ogonek" OutUnicode.ogonek 0x2DB ;
-def_diacritic "\\r"  "ringabove" OutUnicode.ring  0x2DA ;
-def_diacritic "\\v"  "caron" OutUnicode.caron 0x2C7 ;
-def_diacritic "\\textcircled" "circled" OutUnicode.circled 0x25EF ;
+  def_diacritic "\\u"  "breve" OutUnicode.breve breve_alone ;
+  def_diacritic "\\."  "dotabove" OutUnicode.dotabove dotabove_alone ;
+  def_diacritic "\\d"  "dotbelow" OutUnicode.dotbelow dotbelow_alone ;
+  def_diacritic "\\b"  "linebelow" OutUnicode.linebelow  linebelow_alone ;
+  def_diacritic "\\k"  "ogonek" OutUnicode.ogonek ogonek_alone ;
+  def_diacritic "\\r"  "ringabove" OutUnicode.ring ring_alone ;
+  def_diacritic "\\v"  "caron" OutUnicode.caron  caron_alone ;
+  def_diacritic "\\textcircled" "circled" OutUnicode.circled circled_alone ;
 (* activated by amssymb *)
-def_diacritic "\\@doublestruck" "doublestruck" OutUnicode.doublestruck 0 ;
-()
+  def_diacritic "\\@doublestruck" "doublestruck" OutUnicode.doublestruck null ;
+  ()
 ;;
 
 (*
@@ -181,7 +183,7 @@ def_code "\\process@delim@four"
 let int_sup_sub lexbuf =
   let n = Get.get_int (save_arg lexbuf) in
   if !display then begin
-    let {limits=limits ; sup=sup ; sub=sub} = save_sup_sub lexbuf in
+    let {limits=_limits ; sup=sup ; sub=sub} = save_sup_sub lexbuf in
     Dest.int_sup_sub false n
       (scan_this_arg main) (fun () -> ()) sup sub true
   end
@@ -197,28 +199,39 @@ def_code "\\@addvsize"
     Dest.addvsize n)
 ;;
 
-(* Various outworld information *)
+(*****************)
+(* Cummunication *)
+(*****************)
+
+def_code "\\typeout"
+  (fun lexbuf ->
+    let what = Scan.get_prim_arg lexbuf in
+    prerr_endline what )
+;;
+
+def_code "\\hva@warn"
+  (fun lexbuf ->
+    let what = Subst.subst_arg lexbuf in
+    warning what )
+;;
+
 def_code "\\@lexbuf"
   (fun _ ->
-    prerr_endline ("LEXBUF: "^string_of_int (Stack.length stack_lexbuf)))
+    prerr_endline ("LEXBUF: "^string_of_int (MyStack.length stack_lexbuf)))
 ;;
 
 def_code "\\@macros"
   (fun _ -> Latexmacros.pretty_table ())
 ;;
 
-let translate_put s =
-  for k=0 to String.length s-1 do
-    Scan.translate_put_unicode s.[k]
-  done
-;;
 
 let def_print name s =
-  def_code name (fun _ -> translate_put s)
+  def_code name (fun _ -> Scan.translate_put_unicode_string s)
 ;;
 
 def_print "\\@basein" Parse_opts.base_in ;
 def_print "\\jobname" Parse_opts.base_out ;
+def_print "\\jobname@base" (Filename.basename Parse_opts.base_out) ;
 def_print "\\@heveacomline"
   (Array.fold_right
      (fun arg r -> arg^" "^r)
@@ -233,7 +246,16 @@ def_code "\\@heveaverbose"
     Misc.verbose := lvl)
 ;;
 
+
+def_code "\\typemacro"
+  (fun lexbuf ->
+    let name = Scan.get_csname lexbuf in
+    let pat,body = Latexmacros.find name in
+    Latexmacros.pretty_macro pat body)
+;;
+
 (* External style-sheet *)
+
 def_code "\\hva@dump@css"
   (fun _ ->
      let name = match Parse_opts.base_out with
@@ -257,30 +279,19 @@ def_code "\\hva@dump@css"
 ;;
 
 
-(* Stacks of command definitions *)
+(* A few subst definitions, with 2 optional arguments *)
 
-def_code "\\@hva@newstack"
-  (fun lexbuf ->
-     let name = get_prim_arg lexbuf in
-     let stack = Stack.create name in
-     def_code
-       ("\\@push"^name)
-       (fun lexbuf ->
-         let cmd = Scan.get_csname lexbuf in
-         let def = Latexmacros.find cmd in
-         Stack.push stack def) ;
-     def_code
-       ("\\@pop"^name)
-       (fun lexbuf ->
-         let cmd = Scan.get_csname lexbuf in
-         try
-           let pat,body = Stack.pop stack in
-           Latexmacros.def cmd pat body
-         with Fatal _ ->
-           warning (Printf.sprintf "Pop empty stack '%s'" name)))
+def "\\makebox" (latex_pat ["" ; ""] 3)
+    (Subst "\\hva@warn{makebox}\\mbox{#3}") ;
+def "\\framebox" (latex_pat ["" ; ""] 3)
+    (Subst "\\hva@warn{framebox}\\fbox{#3}")
 ;;
 
+(*********************)
 (* 'Token' registers *)
+(*********************)
+
+
 def_code "\\newtokens"
   (fun lexbuf ->
     let toks = Scan.get_csname lexbuf in
@@ -296,13 +307,12 @@ let get_tokens toks = match Latexmacros.find_fail toks with
 
 def_code "\\resettokens"
   (fun lexbuf ->
-    let toks = Scan.get_csname lexbuf in    
-    begin try
+    let toks = Scan.get_csname lexbuf in 
+    try
       ignore (get_tokens toks) ;
       Latexmacros.def toks zero_pat (Toks [])
     with Failed ->
-      Misc.warning ("\\resettokens for "^toks^" failed")
-    end)
+      Misc.warning ("\\resettokens for "^toks^" failed"))
 ;;
 
 def_code "\\addtokens"
@@ -342,14 +352,57 @@ def_code "\\appendtokens"
    end)
 ;;
 
-def_code "\\typemacro"
+(* Useful ???
+def_code "\\lrtokens"
   (fun lexbuf ->
-    let name = Scan.get_csname lexbuf in
-    let pat,body = Latexmacros.find name in
-    Latexmacros.pretty_macro pat body)
-;;
+    let toks = Scan.get_csname lexbuf in
+    let out = Out.create_buff () in
 
-(* See also the lrtokens env in latexscan.mll *)
+    let kont =
+      let once = ref false in
+      (fun lexbuf ->
+        if not !once then begin
+          once := true ;
+          begin try match Latexmacros.find_fail toks with
+          | _,Toks l ->
+              let arg = Out.to_string out in
+              Latexmacros.def toks zero_pat (Toks (l@[arg]))
+          | _ -> raise Failed
+          with Failed ->
+            Misc.warning ("\\lrtokens for "^toks^" failed")
+          end
+        end ;
+        main lexbuf) in
+    start_other_scan "lrtokens" (copy kont "lrtokens" out) lexbuf)
+
+;;
+*)
+
+
+(*********************************)
+(* Stacks of command definitions *)
+(*********************************)
+
+def_code "\\hva@newstack"
+  (fun lexbuf ->
+     let name = get_prim_arg lexbuf in
+     let stack = MyStack.create name in
+     def_code
+       ("\\@push"^name)
+       (fun lexbuf ->
+         let cmd = Scan.get_csname lexbuf in
+         let def = Latexmacros.find cmd in
+         MyStack.push stack def) ;
+     def_code
+       ("\\@pop"^name)
+       (fun lexbuf ->
+         let cmd = Scan.get_csname lexbuf in
+         try
+           let pat,body = MyStack.pop stack in
+           Latexmacros.def cmd pat body
+         with Fatal _ ->
+           warning (Printf.sprintf "Pop empty stack '%s'" name)))
+;;
 
 let call_subst lexbuf =
   let csname = get_csname lexbuf in
@@ -505,6 +558,17 @@ register_init "index"
         let theanchor = get_prim_arg lexbuf in
         Index.treat_anchor tag arg theref theanchor) ;
 
+    (* Special indexwrite where the key is given as fst argument and fully
+       processed *)
+    def_code "\\@@@indexwrite"
+      (fun lexbuf ->
+        let tag = get_prim_opt "default" lexbuf in
+        let key = get_prim_arg lexbuf in
+        let nice = Subst.subst_arg lexbuf in
+        let theref = get_prim_arg lexbuf in
+        let arg = key ^ "@" ^ nice in
+        let lbl = Index.treat  tag arg theref in
+        Dest.put lbl) ;
     def_code "\\@printindex"
       (fun lexbuf ->
         let tag =  get_prim_opt "default" lexbuf in
@@ -635,7 +699,7 @@ register_init "xspace"
       try match Lexstate.full_peek_char lexbuf with
       | '{'|'}'|'~'|'.'|'!'|','|':'|'?'|'/'|'\''|')'|'-'
       | ' '|'\t'|'\n' -> ()
-      |  _ -> Dest.put_char ' '
+      |  _c -> Dest.put_char ' '
       with Not_found ->
         warning "\\xspace could not reach next char"))
 ;;
@@ -709,7 +773,7 @@ register_init "graphics"
 (* url package *)
 let verb_arg lexbuf =
   let {arg=url} = save_verbatim lexbuf in
-  translate_put url
+  Scan.translate_put_unicode_string url
 ;;
 
 def_code "\\@verbarg" verb_arg ;
@@ -854,19 +918,21 @@ let do_setkey lexbuf =
       let {arg=key} = save_arg_with_delim "=" xbuff in
       let {arg=value} = save_arg_with_delim "=" xbuff in
       if !verbose > 1 then
-        Printf.fprintf stderr "SETKEY, key=%s, value=%s\n" key value ;
-      let csname = keyval_name family key in
-      if Latexmacros.exists csname then begin
-        if value <> "" then
-          scan_this main (csname^"{"^value^"}")
-        else
-          let defname = csname^"@default" in
-          if Latexmacros.exists defname then
-            scan_this main defname
+        Printf.fprintf stderr "SETKEY, key='%s', value='%s'\n" key value ;
+      if key <> "" then begin
+        let csname = keyval_name family key in
+        if Latexmacros.exists csname then begin
+          if value <> "" then
+            scan_this main (csname^"{"^value^"}")
           else
-            warning ("keyval, no default value for key: ``"^key^"''")
-      end else
-        warning ("keyval, uknown key: ``"^key^"''") ;
+            let defname = csname^"@default" in
+            if Latexmacros.exists defname then
+              scan_this main defname
+            else
+              warning ("keyval, no default value for key: '"^key^"'")
+        end else
+          warning ("keyval, unknown key: '"^key^"'")
+      end ;
       do_rec ()
     end in
   do_rec ()
@@ -901,7 +967,9 @@ register_init "natbib"
         let year = subst_arg lexbuf in
         let long = subst_arg lexbuf in
         Auxx.bwrite key
-          ("{"^num^"}{"^auth^"}{"^year^"}{"^long^"}")) ;
+          (Printf.sprintf
+             "{%s}{%s}{{%s}}{{%s}}"
+             num year auth long)) ;
     def_code "\\NAT@bibread"
       (fun lexbuf ->
         let key = get_raw lexbuf in
@@ -940,7 +1008,7 @@ register_init "german"
             | 'a' | 'A' | 'e' | 'E' | 'i' | 'I' | 'o' | 'O' | 'u'| 'U' ->
                 begin try gput_unicode lexbuf (OutUnicode.diaeresis c)
                 with OutUnicode.CannotTranslate -> assert false end
-            | 's'|'z' -> gput_unicode lexbuf 0xDF
+            | 's'|'z' -> gput_unicode lexbuf eszett
             | 'c'|'f'|'l'|'m'|'p'|'r'|'t' as c ->
                 gput lexbuf c (* for "ck and "ff etc. *)
             | 'S' ->
