@@ -9,7 +9,7 @@
 (*                                                                     *)
 (***********************************************************************)
 
-let header = "$Id: text.ml,v 1.79 2006-10-10 11:02:04 maranget Exp $"
+let header = "$Id: text.ml,v 1.83 2007/03/09 13:23:53 maranget Exp $"
 
 
 open Misc
@@ -17,7 +17,7 @@ open Parse_opts
 open Element
 open Lexstate
 open Latexmacros
-open Stack
+open MyStack
 open Length
 
 exception Error of string;;
@@ -48,7 +48,7 @@ let push_out s (a,b,c) = push s (Normal (a,b,c))
 ;;
 
 let pretty_stack s =
-  Stack.pretty
+  MyStack.pretty
    (function
      | Normal (s,args,_) -> "["^s^"]-{"^args^"} "
      | Freeze _   -> "Freeze ") s
@@ -61,7 +61,7 @@ let rec pop_out s = match pop s with
 
 let free_list = ref [];;
 
-let out_stack = Stack.create "out_stack";;
+let out_stack = MyStack.create "out_stack";;
 
 let pblock () =
   if empty out_stack then "" else
@@ -112,14 +112,14 @@ let newstatus nostyle _ a t = match !free_list with
     e
 ;;
 
-type saved_out = status * stack_item Stack.saved
+type saved_out = status * stack_item MyStack.saved
 
-let save_out () = !cur_out, Stack.save out_stack
+let save_out () = !cur_out, MyStack.save out_stack
 
 and restore_out (a,b) =
   if !cur_out != a then begin
     free !cur_out ;
-    Stack.finalize out_stack
+    MyStack.finalize out_stack
       (function
         | Normal (_,_,out) -> out == a
         | _ -> false)
@@ -128,7 +128,7 @@ and restore_out (a,b) =
         | _ -> ())
   end ;  
   cur_out := a ;
-  Stack.restore out_stack b
+  MyStack.restore out_stack b
 
 
 type align_t = Left | Center | Right
@@ -224,68 +224,68 @@ and set_flags f {
 
 
 type stack_t = {
-  s_nitems : int Stack.t ;
-  s_dt : string Stack.t ;
-  s_dcount : string Stack.t ;
-  s_x : (int * int * int * int * int * int) Stack.t ;
-  s_align : align_t Stack.t ;
-  s_in_align : bool Stack.t ;
-  s_underline : string Stack.t ;
-  s_nocount : bool Stack.t ;
-  s_in_table : bool Stack.t ;
-  s_vsize : int Stack.t ;
-  s_active : Out.t Stack.t ;
-  s_pending_par : int option Stack.t ;
-  s_after : (string -> string) Stack.t
+  s_nitems : int MyStack.t ;
+  s_dt : string MyStack.t ;
+  s_dcount : string MyStack.t ;
+  s_x : (int * int * int * int * int * int) MyStack.t ;
+  s_align : align_t MyStack.t ;
+  s_in_align : bool MyStack.t ;
+  s_underline : string MyStack.t ;
+  s_nocount : bool MyStack.t ;
+  s_in_table : bool MyStack.t ;
+  s_vsize : int MyStack.t ;
+  s_active : Out.t MyStack.t ;
+  s_pending_par : int option MyStack.t ;
+  s_after : (string -> string) MyStack.t
 } 
 
 let stacks = {
-  s_nitems = Stack.create "nitems" ;
-  s_dt = Stack.create "dt" ;
-  s_dcount = Stack.create "dcount" ;
-  s_x = Stack.create "x" ;
-  s_align = Stack.create "align" ;
-  s_in_align = Stack.create "in_align" ;
-  s_underline = Stack.create "underline" ;
-  s_nocount = Stack.create "nocount" ;
-  s_in_table = Stack.create "in_table" ;
-  s_vsize = Stack.create "vsize" ;
-  s_active = Stack.create "active" ;
-  s_pending_par = Stack.create "pending_par" ;
-  s_after = Stack.create "after"
+  s_nitems = MyStack.create "nitems" ;
+  s_dt = MyStack.create "dt" ;
+  s_dcount = MyStack.create "dcount" ;
+  s_x = MyStack.create "x" ;
+  s_align = MyStack.create "align" ;
+  s_in_align = MyStack.create "in_align" ;
+  s_underline = MyStack.create "underline" ;
+  s_nocount = MyStack.create "nocount" ;
+  s_in_table = MyStack.create "in_table" ;
+  s_vsize = MyStack.create "vsize" ;
+  s_active = MyStack.create "active" ;
+  s_pending_par = MyStack.create "pending_par" ;
+  s_after = MyStack.create "after"
 } 
 
 type saved_stacks = {
-  ss_nitems : int Stack.saved ;
-  ss_dt : string Stack.saved ;
-  ss_dcount : string Stack.saved ;
-  ss_x : (int * int * int * int * int * int) Stack.saved ;
-  ss_align : align_t Stack.saved ;
-  ss_in_align : bool Stack.saved ;
-  ss_underline : string Stack.saved ;
-  ss_nocount : bool Stack.saved ;
-  ss_in_table : bool Stack.saved ;
-  ss_vsize : int Stack.saved ;
-  ss_active : Out.t Stack.saved ;  
-  ss_pending_par : int option Stack.saved ;
-  ss_after : (string -> string) Stack.saved
+  ss_nitems : int MyStack.saved ;
+  ss_dt : string MyStack.saved ;
+  ss_dcount : string MyStack.saved ;
+  ss_x : (int * int * int * int * int * int) MyStack.saved ;
+  ss_align : align_t MyStack.saved ;
+  ss_in_align : bool MyStack.saved ;
+  ss_underline : string MyStack.saved ;
+  ss_nocount : bool MyStack.saved ;
+  ss_in_table : bool MyStack.saved ;
+  ss_vsize : int MyStack.saved ;
+  ss_active : Out.t MyStack.saved ;  
+  ss_pending_par : int option MyStack.saved ;
+  ss_after : (string -> string) MyStack.saved
 } 
 
 let save_stacks () =
 {
-  ss_nitems = Stack.save stacks.s_nitems ;
-  ss_dt = Stack.save stacks.s_dt ;
-  ss_dcount = Stack.save stacks.s_dcount ;
-  ss_x = Stack.save stacks.s_x ;
-  ss_align = Stack.save stacks.s_align ;
-  ss_in_align = Stack.save stacks.s_in_align ;
-  ss_underline = Stack.save stacks.s_underline ;
-  ss_nocount = Stack.save stacks.s_nocount ;
-  ss_in_table = Stack.save stacks.s_in_table ;
-  ss_vsize = Stack.save stacks.s_vsize ;
-  ss_active = Stack.save stacks.s_active ;
-  ss_pending_par = Stack.save stacks.s_pending_par ;
-  ss_after = Stack.save stacks.s_after
+  ss_nitems = MyStack.save stacks.s_nitems ;
+  ss_dt = MyStack.save stacks.s_dt ;
+  ss_dcount = MyStack.save stacks.s_dcount ;
+  ss_x = MyStack.save stacks.s_x ;
+  ss_align = MyStack.save stacks.s_align ;
+  ss_in_align = MyStack.save stacks.s_in_align ;
+  ss_underline = MyStack.save stacks.s_underline ;
+  ss_nocount = MyStack.save stacks.s_nocount ;
+  ss_in_table = MyStack.save stacks.s_in_table ;
+  ss_vsize = MyStack.save stacks.s_vsize ;
+  ss_active = MyStack.save stacks.s_active ;
+  ss_pending_par = MyStack.save stacks.s_pending_par ;
+  ss_after = MyStack.save stacks.s_after
 }
 
 and restore_stacks 
@@ -304,24 +304,24 @@ and restore_stacks
   ss_pending_par = saved_pending_par ;
   ss_after = saved_after
 } =
-  Stack.restore stacks.s_nitems saved_nitems ;
-  Stack.restore stacks.s_dt saved_dt ;
-  Stack.restore stacks.s_dcount saved_dcount ;
-  Stack.restore stacks.s_x saved_x ;
-  Stack.restore stacks.s_align saved_align ;
-  Stack.restore stacks.s_in_align saved_in_align ;
-  Stack.restore stacks.s_underline saved_underline ;
-  Stack.restore stacks.s_nocount saved_nocount ;
-  Stack.restore stacks.s_in_table saved_in_table ;
-  Stack.restore stacks.s_vsize saved_vsize ;
-  Stack.restore stacks.s_active saved_active ;
-  Stack.restore stacks.s_pending_par saved_pending_par ;
-  Stack.restore stacks.s_after saved_after
+  MyStack.restore stacks.s_nitems saved_nitems ;
+  MyStack.restore stacks.s_dt saved_dt ;
+  MyStack.restore stacks.s_dcount saved_dcount ;
+  MyStack.restore stacks.s_x saved_x ;
+  MyStack.restore stacks.s_align saved_align ;
+  MyStack.restore stacks.s_in_align saved_in_align ;
+  MyStack.restore stacks.s_underline saved_underline ;
+  MyStack.restore stacks.s_nocount saved_nocount ;
+  MyStack.restore stacks.s_in_table saved_in_table ;
+  MyStack.restore stacks.s_vsize saved_vsize ;
+  MyStack.restore stacks.s_active saved_active ;
+  MyStack.restore stacks.s_pending_par saved_pending_par ;
+  MyStack.restore stacks.s_after saved_after
 
 let check_stack what =
-  if not (Stack.empty what)  && not !silent then begin
+  if not (MyStack.empty what)  && not !silent then begin
     prerr_endline
-      ("Warning: stack "^Stack.name what^" is non-empty in Html.finalize") ;
+      ("Warning: stack "^MyStack.name what^" is non-empty in Html.finalize") ;
   end
 ;;
 
@@ -355,6 +355,7 @@ let check_stacks () = match stacks with
   check_stack pending_par ;
   check_stack after
 
+(* Buffer for one line *)
 let line = String.create (!Parse_opts.width +2);;
 
 type saved = string * flags_t * saved_stacks * saved_out
@@ -373,13 +374,13 @@ and hot (l,f,s,o) =
   restore_out o
 
 let stop () =
-  Stack.push stacks.s_active !cur_out.out ;
-  Stack.push stacks.s_pending_par flags.pending_par ;
+  MyStack.push stacks.s_active !cur_out.out ;
+  MyStack.push stacks.s_pending_par flags.pending_par ;
   !cur_out.out <- Out.create_null ()
 
 and restart () =
-  !cur_out.out <- Stack.pop stacks.s_active ;
-  flags.pending_par <- Stack.pop stacks.s_pending_par
+  !cur_out.out <- MyStack.pop stacks.s_active ;
+  flags.pending_par <- MyStack.pop stacks.s_pending_par
 
 let do_do_put_char c =
   Out.put_char !cur_out.out c;;
@@ -446,13 +447,14 @@ let do_flush () =
   flags.x <- -1;
 ;;
   
-let do_put_char_format c =
+let do_put_char_format nbsp c =
   if !verbose > 3 then
     prerr_endline ("caracters read : '"^Char.escaped c^"', x="^string_of_int flags.x^", length ="^string_of_int (flags.hsize));
 
-  if c=' ' then  flags.last_space <- flags.x;
+  if not nbsp && c=' ' then  flags.last_space <- flags.x;
   if flags.x =(-1) then begin
     (* La derniere ligne finissait un paragraphe : on indente *)
+(*    Printf.eprintf "FIRST LINE: %i %i\n" flags.x_start flags.first_line ; *)
     flags.x<-flags.x_start + flags.first_line;   
     for i = 0 to flags.x-1 do
       line.[i]<-' ';
@@ -512,12 +514,21 @@ let do_put_char_format c =
   end;
 ;;  
 
+let direct_output () =  !cur_out.temp || (Out.is_null !cur_out.out) 
+  
 let do_put_char c =
   if !verbose>3 then
     prerr_endline ("put_char:|"^String.escaped (String.make 1 c)^"|");
-  if !cur_out.temp || (Out.is_null !cur_out.out) 
-  then do_do_put_char c
-  else do_put_char_format c
+  if direct_output () then
+    do_do_put_char c
+  else
+    do_put_char_format false c
+
+and do_put_nbsp () =
+  if direct_output () then
+    do_do_put_char ' '
+  else
+    do_put_char_format true ' '
 ;;
 
 let finit_ligne () =
@@ -565,8 +576,7 @@ let is_list = function
 
 let get_fontsize () = 3;;
 
-let nostyle () =
-  !cur_out.nostyle<-true
+let nostyle () = !cur_out.nostyle<-true
 ;;
 
 let clearstyle () =
@@ -575,7 +585,7 @@ let clearstyle () =
 
 let open_mod m =
   if m=(Style "CODE") then begin 
-    do_put "`";
+    do_put "'";
     !cur_out.active <- m::!cur_out.active
   end;
 ;;
@@ -594,6 +604,8 @@ let close_mod () = match !cur_out.active with
 | _ -> ()
 ;;
 
+let has_mod _ = false (* No text-level elements in text *)
+
 let erase_mods _ = ()
 ;;
 
@@ -666,7 +678,7 @@ let do_pending () =
 let try_open_block s args =
   (* Prepare l'environnement specifique au bloc en cours *)
   if !verbose > 2 then
-    prerr_endline ("=> try_open ``"^s^"''");
+    prerr_endline ("=> try_open '"^s^"'");
 
   push stacks.s_x
     (flags.hsize,flags.x,flags.x_start,flags.x_end,
@@ -680,7 +692,7 @@ let try_open_block s args =
     push stacks.s_nitems flags.nitems;
     flags.nitems <- 0;
     flags.x_start <- flags.x_start + 3;
-    flags.first_line <- -2;    
+    flags.first_line <- -2 ;    
     flags.hsize <- flags.x_end - flags.x_start+1;
     
     if not flags.in_align then begin
@@ -873,10 +885,14 @@ let do_item isnum =
   if flags.nitems = 0 then begin let _ = forget_par () in () end ;
   try_flush_par () ;
   flags.nitems<-flags.nitems+1;
-  if isnum then
-    do_put ("\n"^(string_of_int flags.nitems)^". ")
-  else
-    do_put "\n- "
+  do_put_char '\n' ;
+  let saved = flags.first_line in
+  flags.first_line <- 0 ;
+  let tag =
+    if isnum then string_of_int flags.nitems^". "
+    else "- " in
+  do_put tag ;
+  flags.first_line <- saved
 ;;
 
 let item _ = do_item false
@@ -956,9 +972,8 @@ let put_char c =
 ;;
 
 let put_unicode i =
-  if !verbose > 3 then Printf.fprintf stderr "put_unicode: %04X\n" i ;
   do_pending ();
-  try do_put_char (OutUnicode.translate_out i)
+  try OutUnicode.translate_out i do_put_char
   with OutUnicode.CannotTranslate -> raise Misc.CannotPut
 ;;
 
@@ -1018,15 +1033,13 @@ let put_separator () = put " "
 let put_tag _ = ()
 ;;
 
-let put_nbsp () =  put " "
+let put_nbsp () =  do_pending (); do_put_nbsp() 
 ;;
 
-let put_open_group () =
-  ()
+let put_open_group () = ()
 ;;
 
-let put_close_group () =
-  ()
+let put_close_group () = ()
 ;;
 
 let put_in_math s =
@@ -1130,12 +1143,12 @@ let table =  ref {
 } 
 ;;
 
-let table_stack = Stack.create "table_stack";;
-let row_stack = Stack.create "row_stack";;
-let cell_stack = Stack.create "cell_stack";;
+let table_stack = MyStack.create "table_stack";;
+let row_stack = MyStack.create "row_stack";;
+let cell_stack = MyStack.create "cell_stack";;
 
 let multi = ref []
-and multi_stack = Stack.create "multi_stack";;
+and multi_stack = MyStack.create "multi_stack";;
 
 
 let open_table _ _ =
